/*

Copyright 2016 Kai Nagel

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "xmlcplist.h"

bool xmlCplist::hasSubtitles() {
    for (int i = 0; i < ReelList.size(); ++i) {
        xmlReel reel = ReelList[i];
        if (reel.getMainSubtitle_Id() != "") return (true);
    }
    return(false);
}

QString  xmlCplist::writeSubtitleFile(QString ExportFilename, QString DCPDir, QList<xmlAssetmap::xmlAsset> &AssetList) {

    int counter = 1;
    int msOffset = 0;

    QFile fileOut(ExportFilename);
    if (!fileOut.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        return -1;
    }

    QTextStream streamFileOut(&fileOut);
    streamFileOut.setCodec("UTF-8");
    streamFileOut.setGenerateByteOrderMark(true);

//    streamFileOut << "Bla Bla";

    for (int i = 0; i < ReelList.size(); ++i) {
        xmlReel reel = ReelList[i];

        if (reel.getMainSubtitle_Id() != "")
        {
            for (int i=0; i < AssetList.size(); i++){
                xmlAssetmap::xmlAsset asset = AssetList[i];
                if (asset.Id == reel.getMainSubtitle_Id())
                {
                    if (reel.getMainSubtitle_EntryPoint() != 0)
                    {
                        msgBox.setText("Subtilte Offset not implemented.\nPlease contact k.nagel@supramotion.de");
                        msgBox.exec();
                    }
                    xmlDCSubtitle sub(DCPDir + "\\" + QDir::toNativeSeparators(asset.ChunkList[0].Path));
                    counter = sub.read(counter, msOffset, streamFileOut);
                }
            }
        }

        msOffset += (reel.getDuration().toInt() * 1000) / getEditRate();
    }

    streamFileOut.flush();

    fileOut.close();

    return("Successfull");
}

QString xmlCplist::getContentKind() {
    return(ContentKind);
}

QString xmlCplist::getAnnotationText() {
    return(AnnotationText);
}

int xmlCplist::getReelCount() {
    return(ReelList.size());
}

int xmlCplist::getEditRate() {
    xmlReel reel = ReelList[0];

    int i=0;

    if (reel.getEditRate() == "25 1")
    {
        i = 25;
    }
    else if (reel.getEditRate() == "30 1")
    {
        i = 30;
    }
    else if (reel.getEditRate() == "48 1")
    {
        i = 48;
    }
    else if (reel.getEditRate() == "50 1")
    {
        i = 50;
    }
    else if (reel.getEditRate() == "60 1")
    {
        i = 60;
    }
    else
    {
        i = 24;
    }

    return(i);
}

int xmlCplist::getDuration() {
    int dur = 0;
    for (int i = 0; i < ReelList.size(); ++i) {
        xmlReel reel = ReelList[i];
        dur += reel.getDuration().toInt();
    }
    return(dur);
}

QString xmlCplist::getScreenAR() {
    xmlReel reel = ReelList[0];
    return(reel.getScreenAR());
}



void xmlCplist::openFile(QString filename)
{
    xmlFile.setFileName(filename);
    xmlFile.open(QIODevice::ReadOnly);
    xml.setDevice(&xmlFile);
    xml.readNextStartElement();
}

void xmlCplist::read(){
    //qDebug() << "Cplist read aufgerufen!";

    ReelList.clear();

    while (!xml.atEnd() && !xml.hasError())
    {
        if (!xml.readNextStartElement())
        {
            break;
        }

        if (xml.name() == "Id")
        {
            Id = xml.readElementText();
        }
        else if (xml.name() == "AnnotationText")
        {
            AnnotationText = xml.readElementText();
        }
        else if (xml.name() == "IssueDate")
        {
            IssueDate = xml.readElementText();
        }
        else if (xml.name() == "Issuer")
        {
            Issuer = xml.readElementText();
        }
        else if (xml.name() == "Creator")
        {
            Creator = xml.readElementText();
        }
        else if (xml.name() == "ContentTitleText")
        {
            ContentTitleText = xml.readElementText();
        }
        else if (xml.name() == "ContentKind")
        {
            ContentKind = xml.readElementText();
        }
        else if (xml.name() == "ContentVersion")
        {
//           qDebug() << "ContentVersion reached.";
             while (!xml.atEnd() && !xml.hasError())
            {
                if (!xml.readNextStartElement())
                {
                    break;
                }

                if (xml.name() == "Id")
                {
                    ContentVersion_Id = xml.readElementText();
                }
                else if (xml.name() == "LabelText")
                {
                    ContentVersion_LabelText = xml.readElementText();
                }
                else
                {
                    qDebug() << "ERROR: unexpected element:" << xml.name();
                    return;
                }
            }
        }
        else if (xml.name() == "RatingList")
        {
            xml.skipCurrentElement();
        }
        else if (xml.name() == "ReelList")
        {
//            qDebug() << "ReelList reached.";
            while (!xml.atEnd() && !xml.hasError())
            {
                if (!xml.readNextStartElement())
                {
                    break;
                }

                xmlReel newReel;
                if (! newReel.read(xml))
                {
                    qDebug() << "Reached document end or aborted";
                    return;
                }

                ReelList.append(newReel);
            }

        }
        else if (xml.name() == "Signer")
        {

        }
        else if (xml.name() == "X509Data")
        {

        }
        else if (xml.name() == "X509IssuerSerial")
        {

        }
        else if (xml.name() == "X509IssuerName")
        {

        }
        else
        {
            qDebug() << "ERROR: unexpected element:" << xml.name();
        }
    }
    xmlFile.close();
}

// -------------------------------------------------------------------------------------------------

QString xmlCplist::xmlReel::getEditRate()
{
    return(EditRate);
}

QString xmlCplist::xmlReel::getDuration()
{
    return(Duration);
}

QString xmlCplist::xmlReel::getScreenAR()
{
    return(ScreenAspectRatio);
}

QString xmlCplist::xmlReel::getMainSubtitle_Id() {
    return(MainSubtitle_Id);
}

int xmlCplist::xmlReel::getMainSubtitle_EntryPoint() {
    return(MainSubtitle_EntryPoint.toInt());
}

bool xmlCplist::xmlReel::read(QXmlStreamReader& xml)
{
    Q_ASSERT(xml.name() == "Reel");

    while (!xml.atEnd() && !xml.hasError())
    {
        if (!xml.readNextStartElement())
        {
            break;
        }

        if (xml.name() == "Id")
        {
            Id = xml.readElementText();
        }
        else if (xml.name() == "AnnotationText")
        {
            AnnotationText = xml.readElementText();
        }
        else if (xml.name() == "AssetList")
        {
            while (!xml.atEnd() && !xml.hasError())
            {
                if (!xml.readNextStartElement())
                {
                    break;
                }

                if (xml.name() == "MainPicture")
                {
                    //qDebug() << "MainPicture reached.";
                    while (!xml.atEnd() && !xml.hasError())
                    {
                        if (!xml.readNextStartElement())
                        {
                            break;
                        }

                        if (xml.name() == "Id")
                        {
                            MainPicture_Id = xml.readElementText();
                        }
                        else if (xml.name() == "AnnotationText")
                        {
                            MainPicture_AnnotationText = xml.readElementText();
                        }
                        else if (xml.name() == "EditRate")
                        {
                            EditRate = xml.readElementText();
                        }
                        else if (xml.name() == "IntrinsicDuration")
                        {
                            MainPicture_IntrinsicDuration = xml.readElementText();
                        }
                        else if (xml.name() == "EntryPoint")
                        {
                            MainPicture_EntryPoint = xml.readElementText();
                        }
                        else if (xml.name() == "Duration")
                        {
                            Duration = xml.readElementText();
                        }
                        else if (xml.name() == "KeyId")
                        {
                            temp = xml.readElementText();
//                            if (!EditRate.compare(temp)) break;
                        }
                        else if (xml.name() == "Hash")
                        {
                            MainPicture_Hash = xml.readElementText();
                        }
                        else if (xml.name() == "FrameRate")
                        {
                            FrameRate = xml.readElementText();
                        }
                        else if (xml.name() == "ScreenAspectRatio")
                        {
                            ScreenAspectRatio = xml.readElementText();
                        }
                        else
                        {
                            qDebug() << "ERROR: unexpected MainPicture reel element:" << xml.name();
                            return false;
                        }
                    }
                }
                else if (xml.name() == "MainSound")
                {
                    //qDebug() << "MainSound reached.";
                    while (!xml.atEnd() && !xml.hasError())
                    {
                        if (!xml.readNextStartElement())
                        {
                            break;
                        }

                        if (xml.name() == "Id")
                        {
                            MainSound_Id = xml.readElementText();
                        }
                        else if (xml.name() == "AnnotationText")
                        {
                            MainSound_AnnotationText = xml.readElementText();
                        }
                        else if (xml.name() == "EditRate")
                        {
                            temp = xml.readElementText();
//                            if (!EditRate.compare(temp)) break;
                        }
                        else if (xml.name() == "IntrinsicDuration")
                        {
                            MainSound_IntrinsicDuration = xml.readElementText();
                        }
                        else if (xml.name() == "EntryPoint")
                        {
                            MainSound_EntryPoint = xml.readElementText();
                        }
                        else if (xml.name() == "Duration")
                        {
                            temp = xml.readElementText();
//                            if (!Duration.compare(temp)) break;
                        }
                        else if (xml.name() == "KeyId")
                        {
                            temp = xml.readElementText();
//                            if (!EditRate.compare(temp)) break;
                        }
                        else if (xml.name() == "Hash")
                        {
                            MainSound_Hash = xml.readElementText();
                        }
                        else if (xml.name() == "Language")
                        {
                            MainSound_Language = xml.readElementText();
                        }
                        else
                        {
                            qDebug() << "ERROR: unexpected MainSound reel element:" << xml.name();
                            return false;
                        }
                    }
                }
                else if (xml.name() == "MainSubtitle")
                {
                    //qDebug() << "MainSubtitle reached.";
                    while (!xml.atEnd() && !xml.hasError())
                    {
                        if (!xml.readNextStartElement())
                        {
                            break;
                        }

                        if (xml.name() == "Id")
                        {
                            MainSubtitle_Id = xml.readElementText();
                        }
                        else if (xml.name() == "AnnotationText")
                        {
                            MainSubtitle_AnnotationText = xml.readElementText();
                        }
                        else if (xml.name() == "EditRate")
                        {
                            temp = xml.readElementText();
//                            if (!EditRate.compare(temp)) break;
                        }
                        else if (xml.name() == "IntrinsicDuration")
                        {
                            MainSubtitle_IntrinsicDuration = xml.readElementText();
                        }
                        else if (xml.name() == "EntryPoint")
                        {
                            MainSubtitle_EntryPoint = xml.readElementText();
                        }
                        else if (xml.name() == "Duration")
                        {
                            temp = xml.readElementText();
//                            if (!Duration.compare(temp)) break;
                        }
                        else if (xml.name() == "Hash")
                        {
                            MainSubtitle_Hash = xml.readElementText();
                        }
                        else if (xml.name() == "Language")
                        {
                            MainSubtitle_Language = xml.readElementText();
                        }
                        else
                        {
                            qDebug() << "ERROR: unexpected MainSubtitle reel element:" << xml.name();
                            return false;
                        }
                    }
                }
                else if (xml.name() == "MainStereoscopicPicture")
                {
                    qDebug() << "3D DCP!";
                    while (!xml.atEnd() && !xml.hasError())
                    {
                        if (!xml.readNextStartElement())
                        {
                            break;
                        }

                        if (xml.name() == "Id")
                        {
                            MainPicture_Id = xml.readElementText();
                        }
                        else if (xml.name() == "AnnotationText")
                        {
                            MainPicture_AnnotationText = xml.readElementText();
                        }
                        else if (xml.name() == "EditRate")
                        {
                            EditRate = xml.readElementText();
                        }
                        else if (xml.name() == "IntrinsicDuration")
                        {
                            MainPicture_IntrinsicDuration = xml.readElementText();
                        }
                        else if (xml.name() == "EntryPoint")
                        {
                            MainPicture_EntryPoint = xml.readElementText();
                        }
                        else if (xml.name() == "Duration")
                        {
                            Duration = xml.readElementText();
                        }
                        else if (xml.name() == "KeyId")
                        {
                            temp = xml.readElementText();
//                            if (!EditRate.compare(temp)) break;
                        }
                        else if (xml.name() == "Hash")
                        {
                            MainPicture_Hash = xml.readElementText();
                        }
                        else if (xml.name() == "FrameRate")
                        {
                            FrameRate = xml.readElementText();
                        }
                        else if (xml.name() == "ScreenAspectRatio")
                        {
                            ScreenAspectRatio = xml.readElementText();
                        }
                        else
                        {
                            qDebug() << "ERROR: unexpected MainStereoscopicPicture reel element:" << xml.name();
                            return false;
                        }
                    }

                }
                else
                {
                    qDebug() << "ERROR: unexpected AssetList element:" << xml.name();
                    return false;
                }
            }
        }

        else
        {
            qDebug() << "ERROR: unexpected reel element:" << xml.name();
            return false;
        }
    }

    return true;
}


// -------------------------------------------------------------------------------------------------

