/*

Copyright 2016 Kai Nagel

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "xmlassetmap.h"

xmlAssetmap::xmlAssetmap(QString filename)
{
    xmlFile.setFileName(filename);
    xmlFile.open(QIODevice::ReadOnly);
    xml.setDevice(&xmlFile);
    xml.readNextStartElement();
}

QString xmlAssetmap::getId() {
    return(Id);
}

QString xmlAssetmap::getVolumeCount() {
    return(VolumeCount);
}

QString xmlAssetmap::getIssueDate() {
    return(IssueDate);
}

QString xmlAssetmap::getIssuer() {
    return(Issuer);
}

QString xmlAssetmap::getCreator() {
    return(Creator);
}

QString xmlAssetmap::getPklFilename() {

    for (int i = 0; i < AssetList.size(); ++i) {
        xmlAsset asset = AssetList[i];
        if (asset.HasPackingList) {
            if (asset.ChunkList.size()) {
                return asset.ChunkList.first().Path;
            }
        }
    }

    return "";
}

const QList<xmlAssetmap::xmlAsset>& xmlAssetmap::getAssetList() const
{
    return AssetList;
}



void xmlAssetmap::read(){
    //    qDebug() << "read aufgerufen!";

    while (!xml.atEnd() && !xml.hasError())
    {
        if (!xml.readNextStartElement())
        {
            break;
        }

        if (xml.name() == "Id")
        {
            Id = xml.readElementText();
        }
        else if (xml.name() == "VolumeCount")
        {
            VolumeCount = xml.readElementText();
        }
        else if (xml.name() == "AnnotationText")
        {
            AnnotationText = xml.readElementText();
        }
        else if (xml.name() == "IssueDate")
        {
            IssueDate = xml.readElementText();
        }
        else if (xml.name() == "Issuer")
        {
            Issuer = xml.readElementText();
        }
        else if (xml.name() == "Creator")
        {
            Creator = xml.readElementText();
        }
        else if (xml.name() == "AssetList")
        {
            while (!xml.atEnd() && !xml.hasError())
            {
                if (!xml.readNextStartElement())
                {
                    break;
                }

                if (xml.name() == "Asset")
                {
                    xmlAsset newAsset;
                    if (! newAsset.read(xml))
                    {
                        qDebug() << "Reached document end or aborted";
                        return;
                    }

                    AssetList.append(newAsset);
                }
                else
                {
                    qDebug() << "ERROR: unexpected element:" << xml.name();
                    return;
                }
            }
        }
        else
        {
            qDebug() << "ERROR: unexpected element:" << xml.name();
        }
    }
    xml.clear();
    xmlFile.close();
}


// -------------------------------------------------------------------------------------------------

bool xmlAssetmap::xmlAsset::read(QXmlStreamReader& xml)
{
    Q_ASSERT(xml.name() == "Asset");

    HasPackingList = false;

    while (!xml.atEnd() && !xml.hasError())
    {
        if (!xml.readNextStartElement())
        {
            break;
        }

        if (xml.name() == "Id")
        {
            Id = xml.readElementText();
        }
        else if (xml.name() == "AnnotationText")
        {
            AnnotationText = xml.readElementText();
        }
        else if (xml.name() == "PackingList")
        {
            PackingList = xml.readElementText();
            HasPackingList = true;
        }
        else if (xml.name() == "ChunkList")
        {
            while (!xml.atEnd() && !xml.hasError())
            {
                if (!xml.readNextStartElement())
                {
                    break;
                }

                if (xml.name() == "Chunk")
                {
                    xmlChunk newChunk;
                    if (! newChunk.read(xml))
                    {
                        return false;
                    }

                    ChunkList.append(newChunk);
                }
                else
                {
                    qDebug() << "ERROR: unexpected chunk element:" << xml.name();
                    return false;
                }
            }
        }
        else
        {
            qDebug() << "ERROR: unexpected chunk element:" << xml.name();
            return false;
        }
    }

    return true;
}


// -------------------------------------------------------------------------------------------------

bool xmlAssetmap::xmlChunk::read(QXmlStreamReader& xml)
{
    Q_ASSERT(xml.name() == "Chunk");

    while (!xml.atEnd() && !xml.hasError())
    {
        if (!xml.readNextStartElement())
        {
            break;
        }

        if (xml.name() == "Path")
        {
            Path = xml.readElementText();
            //            qDebug() << "  Chunk path:" << Path;
        }
        else if (xml.name() == "VolumeIndex")
        {
            VolumeIndex = xml.readElementText();
        }
        else if (xml.name() == "Offset")
        {
            Offset = xml.readElementText();
        }
        else if (xml.name() == "Length")
        {
            Length = xml.readElementText();
        }
        else
        {
            qDebug() << "ERROR: unexpected chunk element:" << xml.name();
            return false;
        }
    }

    return true;
}
