/*

Copyright 2016 Kai Nagel

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef XMLASSETMAP_H
#define XMLASSETMAP_H

#include <QFile>
#include <QXmlStreamReader>
#include <QDebug>
#include <QString>
#include <QList>

class xmlAssetmap
{
public:
    class xmlAsset;

    xmlAssetmap(QString filename);
    void read();

    QString getId(), getVolumeCount(), getIssueDate(), getIssuer(), getCreator(), getPklFilename();

    const QList<xmlAsset>& getAssetList() const;

    class xmlChunk
    {
    public:
        bool read(QXmlStreamReader& stream);

        QString Path;
        QString VolumeIndex;
        QString Offset;
        QString Length;
    };

    class xmlAsset
    {
    public:
        bool read(QXmlStreamReader& stream);

        QString Id;
        QString AnnotationText;
        bool HasPackingList;
        QString PackingList;
        QList<xmlChunk> ChunkList;
    };

private:
    QXmlStreamReader xml;
    QFile xmlFile;

    QString Id;
    QString VolumeCount;
    QString AnnotationText;
    QString IssueDate;
    QString Issuer;
    QString Creator;
    QList<xmlAsset> AssetList;
};

#endif // XMLASSETMAP_H
