/*

Copyright 2016 Kai Nagel

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef XMLCPLIST_H
#define XMLCPLIST_H

#include <QFile>
#include <QXmlStreamReader>
#include <QDebug>
#include <QString>
#include <QList>
#include <QFileDialog>
#include <QMessageBox>

#include "xmlassetmap.h"
#include "xmldcsubtitle.h"

class xmlCplist
{
public:
    class xmlReel;

    void openFile(QString filename);
    void read();

    QString getContentKind(), getAnnotationText(), getScreenAR();
    int getReelCount(), getEditRate(), getDuration();

    bool hasSubtitles();
    QString writeSubtitleFile(QString ExportFilename, QString DCPDir, QList<xmlAssetmap::xmlAsset> &AssetList);

    const QList<xmlReel>& getReelList() const;

    class xmlReel
    {
    public:
        QString getEditRate();
        QString getDuration();
        QString getScreenAR();
        QString getMainSubtitle_Id();
        int getMainSubtitle_EntryPoint();

        bool read(QXmlStreamReader& stream);

   private:
        QString Id;
        QString AnnotationText;
        QString EditRate;
        QString Duration;
        QString FrameRate;
        QString ScreenAspectRatio;
        QString MainPicture_Id;
        QString MainPicture_AnnotationText;
        QString MainPicture_IntrinsicDuration;
        QString MainPicture_EntryPoint;
        QString MainPicture_Hash;
        QString MainSound_Id;
        QString MainSound_AnnotationText;
        QString MainSound_IntrinsicDuration;
        QString MainSound_EntryPoint;
        QString MainSound_Hash;
        QString MainSound_Language;
        QString MainSubtitle_Id;
        QString MainSubtitle_AnnotationText;
        QString MainSubtitle_IntrinsicDuration;
        QString MainSubtitle_EntryPoint;
        QString MainSubtitle_Hash;
        QString MainSubtitle_Language;

        QString temp;
    };

private:
    QXmlStreamReader xml;
    QFile xmlFile;

    QMessageBox msgBox;


    QString Id;
    QString AnnotationText;
    QString IssueDate;
    QString Issuer;
    QString Creator;
    QString ContentTitleText;
    QString ContentKind;
    QString ContentVersion_Id;
    QString ContentVersion_LabelText;
    int ReelCount;

    QList<xmlReel> ReelList;
//    QList<xmlAssetmap::xmlAsset> AssetList;





};

#endif // XMLCPLIST_H
