/*

Copyright 2016 Kai Nagel

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "xmldcsubtitle.h"

xmlDCSubtitle::xmlDCSubtitle(QString filename)
{
    xmlFile.setFileName(filename);
    xmlFile.open(QIODevice::ReadOnly);
    xml.setDevice(&xmlFile);
    xml.readNextStartElement();
}

int xmlDCSubtitle::read(int i, int msOffset, QTextStream &streamFileOut){
//    qDebug() << "read aufgerufen!";

    while (!xml.atEnd() && !xml.hasError())
    {
        if (!xml.readNextStartElement())
        {
            break;
        }

        if (xml.name() == "SubtitleID")
        {
            SubtitleID = xml.readElementText();
        }
        else if (xml.name() == "MovieTitle")
        {
            MovieTitle = xml.readElementText();
        }
        else if (xml.name() == "ReelNumber")
        {
            ReelNumber = xml.readElementText();
        }
        else if (xml.name() == "Language")
        {
            Language = xml.readElementText();
        }
        else if (xml.name() == "LoadFont")
        {
            xml.skipCurrentElement();
        }
        else if (xml.name() == "Font")
        {
            QString Color = xml.attributes().value("Color").toString();

            Color.remove(0,2);

            FontOpen = "<font color=#" + Color + ">";
            FontClose = "</font>";

            QString currentC = Color;
            bool currentI = false;
            bool currentB = false;
            bool currentU = false;


            if (xml.attributes().value("Italic") == "yes")
            {
                FontOpen = FontOpen + "<i>";
                currentI = true;
                FontClose = "</i>" + FontClose;
            }

            if (xml.attributes().value("Weight") == "bold")
            {
                FontOpen = FontOpen + "<b>";
                currentB = true;
                FontClose = "</b>" + FontClose;
            }

            if (xml.attributes().value("Underlined") == "yes")
            {
                FontOpen = FontOpen + "<u>";
                currentU = true;
                FontClose = "</u>" + FontClose;
            }

            while (!xml.atEnd() && !xml.hasError())
            {
                if (!xml.readNextStartElement())
                {
                    break;
                }

                if (xml.name() == "Subtitle")
                {
                    streamFileOut << i << "\n";
                    i++;

                    streamFileOut << TC.dcp2mstc(xml.attributes().value("TimeIn").toString(), msOffset) << " --> " << TC.dcp2mstc(xml.attributes().value("TimeOut").toString(), msOffset) << "\n";

                    while (!xml.atEnd() && !xml.hasError())
                    {
                        if (!xml.readNextStartElement())
                        {
                            break;
                        }

                        if (xml.name() == "Text")
                        {
                            streamFileOut << FontOpen;

                            streamFileOut << xml.readElementText(QXmlStreamReader::IncludeChildElements).simplified();

                            streamFileOut << FontClose << "\n";
                        }
                        else
                        {
                            qDebug() << "ERROR: unexpected element:" << xml.name();
                            return i;
                        }

                    }
                    streamFileOut << "\n";

                }
                else
                {
                    qDebug() << "ERROR: unexpected element:" << xml.name();
                    return i;
                }
            }
        }
        else
        {
            qDebug() << "ERROR: unexpected element:" << xml.name();
        }
    }
    xml.clear();
    xmlFile.close();


    return i;
}
