/*

Copyright 2016 Kai Nagel

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "xmltest.h"
#include "ui_xmltest.h"

XmlTest::XmlTest(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::XmlTest)
{
    ui->setupUi(this);

    ui->lineEditDcpFolder->setText(QDir::toNativeSeparators(QStandardPaths::locate(QStandardPaths::DesktopLocation, QString(), QStandardPaths::LocateDirectory)));
    ui->lineEditExportFilename->setText(QDir::toNativeSeparators(QStandardPaths::locate(QStandardPaths::DesktopLocation, QString(), QStandardPaths::LocateDirectory)));
}

XmlTest::~XmlTest()
{
    delete ui;
}

void XmlTest::on_actionAbout_triggered()
{

    Ui::About uiAboutDialog;
    QDialog a;

    uiAboutDialog.setupUi(&a);
    uiAboutDialog.label_2->setText(Version);

    a.exec();
}

void XmlTest::on_actionReport_an_error_triggered()
{

    Ui::ReportError uiReportError;
    QDialog a;

    uiReportError.setupUi(&a);
    uiReportError.label_2->setText(Version);

    a.exec();
}

void XmlTest::on_actionCheck_for_updates_triggered()
{
    QString url = "https://www.supramotion.de/versioncheckDCP-SubtitleExtract.php?version=" + QString::fromLocal8Bit(Version);
    QDesktopServices::openUrl(url);
}

void XmlTest::on_pushButtonSave_clicked()
{
    ExportFilename = QDir::fromNativeSeparators(ui->lineEditExportFilename->text());
    ExportFilename = QFileDialog::getSaveFileName(this, tr("Export to"), ExportFilename, tr("SubRip (*.srt)")); //;;Spruce Subtitle (*.stl)"));
    ExportFilename = QDir::toNativeSeparators(ExportFilename);

    ui->lineEditExportFilename->setText(ExportFilename);

    cpl.writeSubtitleFile(ExportFilename, BaseDir, AssetList);
}



void XmlTest::on_comboBoxChooseCpl_currentIndexChanged()
{
    if (ui->comboBoxChooseCpl->count() > 0) {
        QString FileCPL = BaseDir + "\\" + pkl.getCplFilenameText(ui->comboBoxChooseCpl->currentIndex() + 1);
        cpl.openFile(FileCPL);
        cpl.read();

        ui->labelCplFilenameText->setText(FileCPL);
        ui->labelContentKindText->setText(cpl.getContentKind());
        ui->labelAnnotationText->setText(cpl.getAnnotationText());
        ui->labelReelCountText->setText(QString::number(cpl.getReelCount()));
        ui->labelEditRateText->setText(QString::number(cpl.getEditRate()));
        ui->labelDurationText->setText(QString::number(cpl.getDuration()) + " (" + TC.frame2tc(cpl.getDuration(), cpl.getEditRate()) + ")");
        ui->labelScreenARText->setText(cpl.getScreenAR());

        if (cpl.hasSubtitles())
        {
            ui->lineEditExportFilename->setEnabled( true );
            ui->pushButtonSave->setEnabled( true );
        }
        else
        {
            ui->lineEditExportFilename->setDisabled( true );
            ui->pushButtonSave->setDisabled( true );
        }
    }
}



void XmlTest::on_pushButtonOpen_clicked()
{
    BaseDir = QDir::fromNativeSeparators(ui->lineEditDcpFolder->text());
    BaseDir = QFileDialog::getExistingDirectory(this, tr("Open Directory"), BaseDir, QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
    BaseDir = QDir::toNativeSeparators(BaseDir);
    ui->lineEditDcpFolder->setText(BaseDir);

    QString FileAssetmap = BaseDir + "/ASSETMAP";
    QFile file(FileAssetmap);

    ui->labelIdText->clear();
    ui->labelVolumeCountText->clear();
    ui->labelIssueDateText->clear();
    ui->labelIssuerText->clear();
    ui->labelCreatorText->clear();
    ui->labelPklFilenameText->clear();
    ui->labelCplCountText->clear();
    ui->comboBoxChooseCpl->clear();

    ui->labelCplFilenameText->clear();
    ui->labelContentKindText->clear();
    ui->labelAnnotationText->clear();
    ui->labelReelCountText->clear();
    ui->labelEditRateText->clear();
    ui->labelDurationText->clear();
    ui->labelScreenARText->clear();

    ui->comboBoxChooseCpl->setDisabled( true );
    ui->lineEditExportFilename->setDisabled( true );
    ui->pushButtonSave->setDisabled( true );

    if (!file.open(QFile::ReadOnly | QFile::Text))
    {
        msgBox.setText("This is no DCP folder or a currently unsupported SMPTE-DCP.\nCan't find ASSETMAP!\n");
        msgBox.exec();
    }
    else
    {
        file.close();
        xmlAssetmap am(FileAssetmap);
        am.read();
        AssetList = am.getAssetList();

        ui->labelIdText->setText(am.getId());
        ui->labelVolumeCountText->setText(am.getVolumeCount());
        ui->labelIssueDateText->setText(am.getIssueDate());
        ui->labelIssuerText->setText(am.getIssuer());
        ui->labelCreatorText->setText(am.getCreator());

        if (am.getPklFilename().size()) {
            QString FilePackingList = BaseDir + "\\" + am.getPklFilename();
//            qDebug() << FilePackingList;
            pkl.reset();
            pkl.openFile(FilePackingList);
            pkl.read();
            ui->labelPklFilenameText->setText(FilePackingList);
            ui->labelCplCountText->setText(QString::number(pkl.getCplCount()));
            for (int i=1; i <= pkl.getCplCount(); i++) {
                ui->comboBoxChooseCpl->addItem(pkl.getCplAnnotationOrFilenameText(i));
            }
            if (pkl.getCplCount() > 1) ui->comboBoxChooseCpl->setEnabled( true );
        }
    }

}
